import xmltodict

from boinc_client.clients.rpc_client import RpcClient
from boinc_client.models.generic_response import GenericResponse
from boinc_client.models.project_attach import ProjectAttachPoll
from boinc_client.models.projects import Projects

EMPTY_PROJECT_LIST = "<projects></projects>"


def all_projects(client: RpcClient) -> dict:
    """Used to get a list of all the projects as found in the all_projects_list.xml file."""
    rpc_resp = client.make_request("<get_all_projects_list/>")
    if not rpc_resp:
        rpc_resp = EMPTY_PROJECT_LIST
    rpc_json = xmltodict.parse(rpc_resp, force_list="project")
    return Projects().load(rpc_json)


def attach_project(
    client: RpcClient, project_name: str, project_url: str, project_key: str
) -> dict:
    """Attach the client to a project."""
    request_xml = f"""<project_attach>
        <project_url>{project_url}</project_url>
        <project_name>{project_name}</project_name>
        <authenticator>{project_key}</authenticator>
    </project_attach>"""
    rpc_resp = client.make_request(request_xml)
    rpc_json = xmltodict.parse(rpc_resp)
    return GenericResponse().load(rpc_json)


def poll_attach_project(client: RpcClient) -> dict:
    """Poll the state of a project attachment."""
    request_xml = "<project_attach_poll/>"
    rpc_resp = client.make_request(request_xml)
    rpc_json = xmltodict.parse(rpc_resp, force_list=("message",))
    return ProjectAttachPoll().load(rpc_json)


def update_project(client: RpcClient, project_url: str) -> dict:
    """Update the state of a project."""
    request_xml = f"""<project_update>
        <project_url>{project_url}</project_url>
    </project_update>"""
    rpc_resp = client.make_request(request_xml)
    rpc_json = xmltodict.parse(rpc_resp)
    return GenericResponse().load(rpc_json)


def detach_project(client: RpcClient, project_url: str) -> dict:
    """Detach from a project."""
    request_xml = f"""<project_detach>
        <project_url>{project_url}</project_url>
    </project_detach>"""
    rpc_resp = client.make_request(request_xml)
    rpc_json = xmltodict.parse(rpc_resp)
    return GenericResponse().load(rpc_json)


def suspend_project(client: RpcClient, project_url: str) -> dict:
    """Suspend a project."""
    request_xml = f"""<project_suspend>
        <project_url>{project_url}</project_url>
    </project_suspend>"""
    rpc_resp = client.make_request(request_xml)
    rpc_json = xmltodict.parse(rpc_resp)
    return GenericResponse().load(rpc_json)


def resume_project(client: RpcClient, project_url: str) -> dict:
    """Resume a project."""
    request_xml = f"""<project_resume>
        <project_url>{project_url}</project_url>
    </project_resume>"""
    rpc_resp = client.make_request(request_xml)
    rpc_json = xmltodict.parse(rpc_resp)
    return GenericResponse().load(rpc_json)



from marshmallow import Schema, fields


class ProjectDiskStats(Schema):
    master_url = fields.Str()
    disk_usage = fields.Float()


class DiskStats(Schema):
    projects = fields.Nested(
        ProjectDiskStats(many=True),
        data_key="project",
        allow_none=True,
        load_default=[],
    )
    d_total = fields.Float()
    d_free = fields.Float()
    d_boinc = fields.Float()
    d_allowed = fields.Float()


class DiskUsage(Schema):
    disk_stats = fields.Nested(DiskStats(), data_key="disk_usage_summary")



from marshmallow import Schema, fields, post_load, pre_load

from boinc_client.models.helpers import (
    create_indexes,
    flatten_data,
    normalise_none_to_list,
    remove_key,
)


class Notice(Schema):
    title = fields.Str(allow_none=True)
    description = fields.Str()
    create_time = fields.Float()
    arrival_time = fields.Float()
    is_private = fields.Bool()
    project_name = fields.Str()
    category = fields.Str()
    link = fields.Url(allow_none=True)
    seqno = fields.Int()

    @post_load
    def _remove_seqno(self, data, **kwargs):
        return remove_key(data, "seqno")


class Notices(Schema):
    notices = fields.Dict(fields.Int(), fields.Nested(Notice()))

    @pre_load
    def _a_normalise_none(self, data, **kwargs):
        return normalise_none_to_list(data, "notices")

    @pre_load
    def _b_flatten_data(self, data, **kwargs):
        return flatten_data(data, "notices", "notice")

    @pre_load
    def _c_create_dict_keys(self, data, **kwargs):
        return create_indexes(data, "notices", "seqno")



from marshmallow import Schema, fields, pre_load

from boinc_client.models.helpers import flatten_data, normalise_none_to_list


class OldResult(Schema):
    project_url = fields.Url()
    result_name = fields.Str()
    app_name = fields.Str()
    exit_status = fields.Int()
    elapsed_time = fields.Float()
    cpu_time = fields.Float()
    completed_time = fields.Float()
    create_time = fields.Float()


class OldResults(Schema):
    old_results = fields.Nested(OldResult(many=True))

    @pre_load
    def _a_normalise_none(self, data, **kwargs):
        return normalise_none_to_list(data, "old_results")

    @pre_load
    def _b_flatten_data(self, data, **kwargs):
        return flatten_data(data, "old_results", "old_result")



from marshmallow import Schema, fields


class Version(Schema):
    major = fields.Int()
    minor = fields.Int()
    patch = fields.Int(data_key="release")


class ClientVersion(Schema):
    version = fields.Nested(Version(), data_key="server_version")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['boinc_client', 'boinc_client.clients', 'boinc_client.models']

package_data = \
{'': ['*']}

install_requires = \
['lxml>=4.9.2,<5.0.0',
 'marshmallow>=3.19.0,<4.0.0',
 'xmltodict>=0.13.0,<0.14.0']

setup_kwargs = {
    'name': 'boinc-client',
    'version': '1.4.4',
    'description': 'Python API for interacting with a BOINC client via RPC',
    'long_description': '# BOINC Client\n\n![Test and Release](https://github.com/SplinterHead/boinc-client/actions/workflows/test-and-release.yml/badge.svg)\n[![boinc-client](https://snyk.io/advisor/python/boinc-client/badge.svg)](https://snyk.io/advisor/python/boinc-client)\n\nPython native library for interacting with a BOINC client via RPC. This library has been designed to have consistent response types.\n\n## Usage\n\n### Setup\n\n```python\nfrom boinc_client import Boinc\nfrom boinc_client.clients.rpc_client import RpcClient\n\n# Hostname or IP of the running BOINC client\nBOINC_HOSTNAME = "192.168.0.2"\n\n# Create an RPC client to connect to the BOINC socket\nrpc_client = RpcClient(hostname=BOINC_HOSTNAME)\nrpc_client.authenticate()\n\n# Create a BOINC client to interact with the RPC socket\nboinc_client = Boinc(rpc_client=rpc_client)\n```\n\n#### RPC Client options\nThe following options can be passed when creating an `RpcClient` instance\n\n| Argument   | Description                                                                | Required | Default |\n|------------|----------------------------------------------------------------------------|----------|---------|\n| `hostname` | Hostname or IP address of the BOINC client                                 | Yes      | None    |\n| `port`     | Exposed port of the BOINC client                                           | No       | 31416   |\n| `timeout`  | Seconds to wait for a successful connection to the RPC socket              | No       | 30      |\n| `password` | Password to authenticate to the BOINC client, required for most operations | No       | None    |\n\n#### Boinc options\nThe following options can be passed when creating a `Boinc` instance\n\n| Argument     | Description                        | Required | Default |\n|--------------|------------------------------------|----------|---------|\n| `rpc_client` | Instance of a configured RpcClient | Yes      | None    |\n\n### Interacting with Boinc\n\n* [Unauthorised Operations](docs/unauthorised.md)\n* [Project Operations](docs/project.md)\n\n## Contributors\n\n<!-- ALL-CONTRIBUTORS-LIST:START - Do not remove or modify this section -->\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable -->\n\n<!-- markdownlint-restore -->\n<!-- prettier-ignore-end -->\n\n<!-- ALL-CONTRIBUTORS-LIST:END -->',
    'author': 'Lewis England',
    'author_email': 'lewis2004@hotmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from marshmallow import Schema, fields, pre_load

from boinc_client.models.project_status import ProjectState
from boinc_client.models.result import Result


class GuiInfo(Schema):
    projects = fields.Nested(ProjectState(many=True), data_key="project")
    results = fields.Nested(Result(many=True), data_key="result")


class SimpleGuiInfo(Schema):
    gui_info = fields.Nested(GuiInfo(), data_key="simple_gui_info")

    @pre_load
    def _a_normalise_none(self, data, **kwargs):
        if not data["simple_gui_info"]:
            data["simple_gui_info"] = {"project": [], "result": []}
        return data
